Attribute VB_Name = "modVB5_m201"
' Lawson Labs, Inc.
' 3239 Phoenixville Pike
' Malvern, PA  19355
' 800 321-5355 or 406 257-5355
' World Wide Web - www.lawsonlabs.com
' Programmer - Tim Van Dusen
' Last modified: 11-10-98
' filename: VB5_M201
' requires NT_M201.DLL and Win95/98/NT

Option Explicit

Type M201
    comPort As Integer           'Serial port COM 1 through 4
    scanInterval As Double       'Scan Interval in seconds
    numChannels As Integer       'Number of channels for multi-channel scan
    tScan As Integer             '0=single chan, 1=normal scan, 2=cal scan
    chan(0 To 5) As Integer      'channel selection codes for multi-chan scan
    singleChan As Integer        'channel selection code for single-chan scan
    lastDigInp As Integer        'last valid digital input word
    version As Integer           'microcode Version, read from Model 201
    channel As Integer           'Current selected channel 0-7
    subChan As Integer           'Current selected sub-channel 0-15
    rawCount As Long             'unscaled A/D count 0 to 16,777,215
    voltage As Double            'voltage in MV
    digOutWord As Long           'digital output word
    wordCnt As Integer           'A/D word count. 2= 16-bit mode, 3= 24-bit
    biPolar As Integer           'Unipolar/bipolar flag. 0= bi, 1= uni
    errorType As Long            'error code from dll GetOneScan function
    filter As Integer            'filter freq. 2= 400Hz, 1= 40, 0= 4Hz
    gain As Integer              'power of 2 for gain. 0=x1, 1=x2, 2=x4, etc
    rate As Single               'internal data rate for A/D
    modeRegLo As Integer         'low byte of compacted A/D mode
    modeRegMid As Integer        'mid byte of compacted A/D mode
    modeRegHi As Integer         'high byte of compacted A/D mode
    baud As Integer              '0= 9600, 1= 4800, 2= 2400 . . . 5 = 300
    sleeping As Integer          'Flag. 1= asleep, 0= awake
    standBy As Integer           'Flag. 1= in standby, 0= operating
    timeBaseCnt As Long          'not used(placeholder only), for Model 202 only
    average As Long              'digital averaging in 201 (powers of 2)
    TXQUEUESIZE As Long          'transmit buffer size (not used).
    RXQUEUESIZE As Long          'recieve buffer size. 16 to 32767
End Type

Public NewScanBuff(300) As Byte  'buffer for passing one scan from DLL
Public cc(6) As Integer          'control code workspace
Public M_201 As M201             'M201 structured data shared with DLL
Public CfgFileSpec As String     'Config file name
Public localAvg As Integer       'Averaging done in host
Public IdComDev As Long          'Handle to Com-port (not for use in this code)
Public retVal As Integer         'return value for DLL functions
Public WaitTime As Double        'filter settling time in seconds
Public Msg As String             'message string
Public fStopScan As Boolean      'flag to stop scanning
Public hNT_m201_dll              'handle to DLL


Public FullScaleOutput As Double
Public CountsPerVolt As Double
Public OutputOffset As Double
Public timeBaseCnt As Double
Public VoltsOut As Double
Public DacCnt() As Double       ' Array to hold counts for desired voltages

Public CntrSelectCmd As Integer
Public CntrWriteCmd As Integer
Public CntrCtrl() As Integer    ' Array to hold counter select codes
Public CntrlInit As Integer
Public CntrRead As Integer
Public NumDACs As Integer
Public CountHi As Integer
Public CountLo As Integer
Public OUTB1 As Integer
Public OUTB2 As Integer
Public DAC As Integer

Public fRetVal As Boolean

'NT_M201.DLL external functions
Declare Function SetPtrs Lib "NT_m201.dll" (ByRef T_M201 As M201, _
            ByRef NewScanBuff As Byte) As Integer

'this group of functions corresponds exactly to commands described in
'the programmmer's reference section of your manual
Declare Function DLL_SendPacket Lib "NT_m201.dll" _
        (ByVal bToken As Integer, ByVal bData As Integer) As Boolean
        
Declare Function RemoteAvg Lib "NT_m201.dll" () As Integer
Declare Function FullScaleCal Lib "NT_m201.dll" () As Integer
Declare Function OffsetCal Lib "NT_m201.dll" () As Integer
Declare Function GetDigIn Lib "NT_m201.dll" () As Integer
Declare Function SetDigOut Lib "NT_m201.dll" (ByVal digOutWord As Long) As Integer
Declare Function GetVer Lib "NT_m201.dll" () As Integer
Declare Function GetConversion Lib "NT_m201.dll" () As Integer
Declare Function SetChanCode Lib "NT_m201.dll" () As Integer
Declare Function StopScanning Lib "NT_m201.dll" () As Integer
Declare Function SendADMode Lib "NT_m201.dll" () As Integer
Declare Function GetCheckSum Lib "NT_m201.dll" () As Integer
Declare Function GotoSleep Lib "NT_m201.dll" () As Integer

'starts type of scan determined by sType
Declare Function StartScanning Lib "NT_m201.dll" () As Integer

'establish communications with Model 201 and send all set-up information
Declare Function SignOn Lib "NT_m201.dll" () As Integer

'don't use in this application
Declare Function CloseDevice Lib "NT_m201.dll" () As Integer

'select chan 7, wait, & do offset cal. Select chan 6, wait & do full scale cal.
Declare Function SysCal Lib "NT_m201.dll" () As Integer

'convert rawcount to volts, normally handled in DLL
Declare Function Counts2Volts Lib "NT_m201.dll" () As Double

'fetch one scan, wait scantime first, if one isn't ready
Declare Function GetOneScan Lib "NT_m201.dll" () As Integer

'verify scan timing
Declare Function CheckScanTime Lib "NT_m201.dll" (ByRef MaxTime As Double, _
            ByRef MinTime As Double) As Integer
            
'gets handle to COM-port used by DLL
Declare Sub GetComHandle Lib "NT_m201.dll" (ByRef IdComDev As Long)

'fetch one conversion, waits COM-timeout determined in DLL, if one isn't ready
Declare Sub Gconv Lib "NT_m201.dll" ()

'remove "size" bytes from serial input buffer(normally used for end-of-scan _
            framing token only)
Declare Sub RemFromBuffer Lib "NT_m201.dll" (ByVal size As Long)

'set pre-filter
Declare Sub SetFilter Lib "NT_m201.dll" (ByVal filter As Integer)

'wait filter delay
Declare Sub FilterDelay Lib "NT_m201.dll" (ByVal filter As Integer)

'Model 202 only
Declare Sub WriteCntr Lib "NT_m201.dll" (ByVal cntPort As Integer, _
            ByVal cntrCount As Long)
            
'Windows API external functions
Declare Function MessageBox Lib "user32" Alias "MessageBoxA" _
    (ByVal hwnd As Long, ByVal lpText As String, ByVal lpCaption As String, _
    ByVal wType As Long) As Long
Declare Function GetModuleHandle Lib "kernel32" Alias "GetModuleHandleA" _
                (ByVal lpModuleName As String) As Long
Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long

'  Configuration file should be created and tested using the C or
'  Quick Basic driver software.

Public Sub ReadCfgFile()
    Dim cf As Long
    Dim sStr As String
    CfgFileSpec = App.Path & "\201ewin.cfg"
    cf = FreeFile
    On Error GoTo NO_FILE
    Open CfgFileSpec For Input As #cf
    Line Input #cf, sStr
    M_201.TXQUEUESIZE = Val(sStr)
    Line Input #cf, sStr
    M_201.RXQUEUESIZE = Val(sStr)
    Line Input #cf, sStr
    M_201.baud = Val(sStr)
    Line Input #cf, sStr
    M_201.comPort = Val(sStr)
    Line Input #cf, sStr
    M_201.scanInterval = Val(sStr)
    Line Input #cf, sStr
    M_201.numChannels = Val(sStr)
    Line Input #cf, sStr
    M_201.average = Val(sStr)
    Line Input #cf, sStr
    M_201.tScan = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(0) = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(1) = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(2) = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(3) = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(4) = Val(sStr)
    Line Input #cf, sStr
    M_201.chan(5) = Val(sStr)
    Line Input #cf, sStr
    M_201.singleChan = Val(sStr)
    Line Input #cf, sStr
    M_201.channel = Val(sStr)
    Line Input #cf, sStr
    M_201.subChan = Val(sStr)
    Line Input #cf, sStr
    M_201.wordCnt = Val(sStr)
    Line Input #cf, sStr
    M_201.biPolar = Val(sStr)
    Line Input #cf, sStr
    M_201.filter = Val(sStr)
    Line Input #cf, sStr
    M_201.gain = Val(sStr)
    Line Input #cf, sStr
    M_201.rate = Val(sStr)
    Line Input #cf, sStr
    M_201.modeRegLo = Val(sStr)
    Line Input #cf, sStr
    M_201.modeRegMid = Val(sStr)
    Line Input #cf, sStr
    M_201.modeRegHi = Val(sStr)
    Line Input #cf, sStr
    localAvg = Val(sStr)
    Close #cf
    M_201.comPort = 1
    GoTo EXIT_READ_CFG
    
NO_FILE:
    MsgDisplay ("ERROR opening 201EWIN.CFG" _
        & Chr(10) _
        & Chr(10) _
        & Chr(10) _
        & "Locate and place configuration file in" _
        & Chr(10) _
        & Chr(10) _
        & "same directory as application and try again." _
        & Chr(10) _
        & Chr(10))
    End
    
EXIT_READ_CFG:
    On Error GoTo 0
End Sub
Public Function InitDACS()
    'All counters are initialized first, each used counter needs it's mode
    ' register set then the count from DacCnt(Dac%) is sent to each the count
    ' represents voltage in all cases except DAC 0 which is the timebase
    ' register. Larger timebases give better resolution. Smaller ones, less
    ' ripple. Change TimebaseCnt and/or NumDacs to suit your needs.
    '******** set these variables as needed ********
    NumDACs = 2             ' Number of DACs in use, 1 to 5.
    FullScaleOutput = 10    ' Set to match your hardware range in volts.
    OutputOffset = 0        ' Set to 5 for +/-5 volt operation
                            '  or 0 for 0 to 5 or 0 to 10 volts.
    timeBaseCnt = 10000     ' Full-scale dac count 2 to 65535.
                            ' One count per mv with timebase of 10000
                            '  and 10 volt range.
                            
    '******** set these variables as needed ********
    CountsPerVolt = timeBaseCnt / FullScaleOutput
                            'number of counts for 1 volt output
    
    CntrSelectCmd = &H9     'command to write selection data to counter
                            'control register. Data is defined below:
    
    '         4-bit control register   bit    3         2        1        0
    '                                     EN 3,4,&5   EN 1&2     A1       A0
    
    CntrlInit = 0           'data to disable counter reads and writes
    
    CntrWriteCmd = &H7      'command to write to counter registers
                            'data is written into selected counter
    
    CntrRead = &HC          'argument for digital input command (&H80)
                            'used to read counters (example not included)
    
    ReDim CntrCtrl(5)       ' Array to hold counter select codes
    ReDim DacCnt(5)         ' Array to hold counts for desired voltages
    DacCnt(0) = timeBaseCnt ' Except this one, which is the timebase
    For DAC = 1 To NumDACs
        DacCnt(DAC) = 1
        frmVB5_M201.ComboDACs.AddItem (DAC)
    Next DAC                'for initialization
    
                            ' Dacs% can be initialized to any desired voltage
                            ' Here, they are set to the minimum ouput.
    
                            'These are constants for addressing the hardware
    CntrCtrl(0) = &H4       'data to address timebase register
    CntrCtrl(1) = &H5       'data to address Dac 1
    CntrCtrl(2) = &H6       'data to address Dac 2
    CntrCtrl(3) = &H8       'data to address Dac 3
    CntrCtrl(4) = &H9       'data to address Dac 4
    CntrCtrl(5) = &HA       'data to address Dac 5
    
                    ' Counter 0 is used to trigger counters 1 through 5
                    ' MODE 2, 10 MHz clock input
                    ' any count from 2 to 65535 can be used
                    ' higher resolution means slower settling
    
                    ' other counters are voltage outputs
                    ' MODE 1, 10 MHz clock, gated by counter 0
                    ' 1 to TIMEBASECNT counts = 0 to 5 or 10 VDC out
                  
    frmVB5_M201.ComboDACs.Text = "1"
    For DAC = 0 To NumDACs              'set modes

      If (DAC = 0) Or (DAC = 3) Then    'select mode register
          OUTB1 = CntrSelectCmd         'mode reg needs to be selected only
          OUTB2 = CntrCtrl(DAC) Or &H3  'once per 3-channel counter
          fRetVal = DLL_SendPacket(OUTB1, OUTB2)
      End If
      OUTB1 = CntrWriteCmd
      If DAC = 0 Then
          OUTB2 = &H34                      'write mode 2
      Else
          OUTB2 = &H32 + DAC * 64           'write mode 1
          If DAC > 2 Then OUTB2 = OUTB2 - 192 'adjustment for 2nd counter IC
      End If
      fRetVal = DLL_SendPacket(OUTB1, OUTB2)
    Next DAC                                'done setting modes
    
    For DAC = 0 To NumDACs                  'now set counts
      OUTB1 = CntrSelectCmd
      OUTB2 = CntrCtrl(DAC)                 'select counter register
      fRetVal = DLL_SendPacket(OUTB1, OUTB2)
      CountHi = Int(DacCnt(DAC) / 256)      ' get hi and lo bytes of count
      CountLo = Fix(DacCnt(DAC) - CountHi * 256!)   ' from DacCnt array
      OUTB1 = CntrWriteCmd
      OUTB2 = CountLo                       ' write data low byte
      fRetVal = DLL_SendPacket(OUTB1, OUTB2)
      OUTB2 = CountHi                       ' write data high byte
      fRetVal = DLL_SendPacket(OUTB1, OUTB2)
    Next DAC
    InitDACS = fRetVal
End Function
Public Function SetDACS()
    If DAC < 1 Or DAC > NumDACs Then
        SetDACS = False
        Return
    End If
    DacCnt(DAC) = (VoltsOut + OutputOffset) * CountsPerVolt
    If (DacCnt(DAC) > timeBaseCnt) Then DacCnt(DAC) = timeBaseCnt
    If DacCnt(DAC) < 1 Then DacCnt(DAC) = 1     ' 0 is illegal count
    fRetVal = UpdateCntr
    SetDACS = fRetVal
End Function
Public Function UpdateCntr()
    OUTB1 = CntrSelectCmd
    OUTB2 = CntrCtrl(DAC)                  'select counter register
    fRetVal = DLL_SendPacket(OUTB1, OUTB2)
    UpdateCntr = fRetVal
    If fRetVal = False Then Return
    CountHi = Int(DacCnt(DAC) / 256)        'get hi and lo bytes of count
    CountLo = Fix(DacCnt(DAC) - CountHi * 256!)
    OUTB1 = CntrWriteCmd
    OUTB2 = CountLo                         'data low
    fRetVal = DLL_SendPacket(OUTB1, OUTB2)
    UpdateCntr = fRetVal
    If fRetVal = False Then Return
    OUTB2 = CountHi                         'data high
    fRetVal = DLL_SendPacket(OUTB1, OUTB2)
    UpdateCntr = fRetVal
End Function
Public Sub Wait(WaitTime)
    Dim dblEndTime As Double
    dblEndTime = Timer + WaitTime
    Do While dblEndTime > Timer
        DoEvents
    Loop
End Sub
Public Sub MsgDisplay(Msg)
    Dim retVal As Long
    'MsgBox Msg, vbOK       'this box has no title control
    retVal = MessageBox(0, Msg, "Error ----", 0)
End Sub


